@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """GitHub class file."""
 
 import requests
@@ -32,15 +33,10 @@ class GitHub(object):
 
         # set the base urls
         self.base_url = 'https://api.github.com'
-        self.org_base_url = '%s/orgs/%s' % (
-            self.base_url,
-            self.org,
-        )
+        self.org_base_url = '{}/orgs/{}'.format(self.base_url, self.org)
 
         # set the headers for authorized requests
-        self.headers = {
-            'Authorization': 'token %s' % (self.token),
-        }
+        self.headers = {'Authorization': 'token {}'.format(self.token)}
 
     #
     # Sub Classes
@@ -63,58 +59,79 @@ class GitHub(object):
     #
     # Helpers
     #
+    def get(self, url, headers={}, params={}):
+        """Return a response from a GET call."""
+        # add any additional headers
+        headers = {**self.headers, **headers}
+
+        # ret response to request
+        response = requests.get(url, headers=headers, params=params)
+
+        # raise for status
+        response.raise_for_status()
+
+        # return json
+        return response
+
     def get_list(self, base_url, url, headers={}, params={}):
-        """Return a list from a GET call."""
+        """Return a paginated list from a GET call."""
         items_list = []
 
         # add any additional headers
-        request_headers = self.headers
-        for header in headers:
-            request_headers[header] = headers[header]
+        headers = {**self.headers, **headers}
 
-        next_url = '%s/%s' % (base_url, url)
+        next_url = '{}/{}'.format(base_url, url)
 
         while next_url:
             # get response to request
-            response = requests.get(
-                next_url,
-                headers=request_headers,
-                params=params,
-            )
+            response = requests.get(next_url, headers=headers, params=params)
+
+            # raise for status
+            response.raise_for_status()
 
             # get next url from response links
             next_url = response.links.get('next', {}).get('url')
 
             # add items to list
-            try:
-                items_list.extend(response.json())
-            except Exception as e:
-                print('ERROR converting response to JSON: %s' % (e))
-                print(response.text)
+            items_list.extend(response.json())
 
         return items_list
 
     #
     # Organizations
     #
-    def get_org(self, org):
+    def get_org(self, org=None):
         """Return an organization."""
         # GET /orgs/:org
-        url = '%s/orgs/%s' % (self.base_url, self.org)
-        return requests.get(url, headers=self.headers).json()
+        if not org:
+            org = self.org
+        url = '{}/orgs/{}'.format(self.base_url, org)
+        return self.get(url).json()
 
     #
     # Org Hooks
     #
     def get_org_hooks(self):
         """Return a list of organization hooks."""
+        # GET /orgs/:org/hooks
         return self.get_list(self.org_base_url, 'hooks')
 
+    #
+    # Org Invitations
+    #
     def get_org_invitations(self):
         """Return a list of organization invitations."""
+        # GET /orgs/:org/invitations
         headers = {'Accept': 'application/vnd.github.dazzler-preview'}
         return self.get_list(self.org_base_url, 'invitations', headers=headers)
 
+    def get_org_invitation_teams(self, invitation_id):
+        """Return a list of organization invitation teams."""
+        # GET /orgs/:org/invitations/:invitation_id/teams
+        headers = {'Accept': 'application/vnd.github.dazzler-preview'}
+        url = 'invitations/{}/teams'.format(invitation_id)
+        return self.get_list(self.org_base_url, url, headers)
+
     #
     # Org Members
     #
@@ -135,19 +152,19 @@ class GitHub(object):
     def get_org_membership(self, member):
         """Return a user's organization membership."""
         # GET /orgs/:org/memberships/:username
-        url = '%s/orgs/%s/memberships/%s' % (self.base_url, self.org, member)
-        return requests.get(url, headers=self.headers).json()
+        url = '{}/orgs/{}/memberships/{}'.format(self.base_url, self.org, member)
+        return self.get(url).json()
 
     def invite_org_member(self, member):
         """Invite a member to the organization."""
         # PUT /orgs/:org/memberships/:username
-        url = '%s/memberships/%s' % (self.org_base_url, member)
+        url = '{}/memberships/{}'.format(self.org_base_url, member)
         return requests.put(url, headers=self.headers).json()
 
     def remove_org_member(self, member):
         """Delete an invitation to the organization."""
         # DELETE /orgs/:org/memberships/:username
-        url = '%s/memberships/%s' % (self.org_base_url, member)
+        url = '{}/memberships/{}'.format(self.org_base_url, member)
         return requests.put(url, headers=self.headers).json()
 
     #
@@ -172,8 +189,8 @@ class GitHub(object):
     def get_org_public_member(self, member):
         """Check if a user is a public member of the org."""
         # GET /orgs/:org/public_members/:username
-        url = '%s/orgs/%s/public_members/%s' % (self.base_url, self.org, member)
-        return requests.get(url, headers=self.headers)
+        url = '{}/orgs/{}/public_members/{}'.format(self.base_url, self.org, member)
+        return self.get(url)
 
     def get_org_public_members(self):
         """Return a list of organization public members."""
@@ -185,6 +202,7 @@ class GitHub(object):
     #
     def get_org_repos(self):
         """Return a list of organization repos."""
+        # GET /orgs/:org/repos
         headers = {'Accept': 'application/vnd.github.baptiste-preview+json'}
         return self.get_list(self.org_base_url, 'repos', headers=headers)
 
@@ -193,6 +211,7 @@ class GitHub(object):
     #
     def get_org_teams(self):
         """Return a list of organization teams."""
+        # GET /orgs/:org/teams
         # add header for Nested Teams API preview
         headers = {'Accept': 'application/vnd.github.hellcat-preview+json'}
         return self.get_list(self.org_base_url, 'teams', headers)
@@ -210,7 +229,7 @@ class GitHub(object):
             else:
                 pid = parent['id']
                 if pid not in organization:
-                    print('ERROR: Parent ID not found: %s' % (pid))
+                    print('ERROR: Parent ID not found: {}'.format(pid))
                 else:
                     organization[pid]['children'].append(team)
         return organization
@@ -220,84 +239,89 @@ class GitHub(object):
     #
     def get_repo(self, repo, etag=None):
         """Return a repo of the organization."""
-        url = '%s/repos/%s/%s' % (self.base_url, self.org, repo)
+        # GET /repos/:owner/:repo
+        url = '{}/repos/{}/{}'.format(self.base_url, self.org, repo)
         headers = self.headers.copy()
         if etag:
             headers['If-None-Match'] = etag
-            return requests.get(url, headers=headers)
-        return requests.get(url, headers=headers).json()
+            return self.get(url, headers=headers)
+        return self.get(url, headers=headers).json()
 
     def get_repo_collaborators(self, repo, affiliation=None):
         """Return a list of repo hook."""
-        url = 'repos/%s/%s/collaborators' % (self.org, repo)
+        # GET /repos/:owner/:repo/collaborators
+        url = 'repos/{}/{}/collaborators'.format(self.org, repo)
         params = {'affiliation': affiliation}
         return self.get_list(self.base_url, url, params=params)
 
     def get_repo_hooks(self, repo):
         """Return a list of repo hook."""
-        url = 'repos/%s/%s/hooks' % (self.org, repo)
+        # GET /repos/:owner/:repo/hooks
+        url = 'repos/{}/{}/hooks'.format(self.org, repo)
         return self.get_list(self.base_url, url)
 
+    def update_repo_hook(self, repo, hook_id, body):
+        """Update a repo hook."""
+        # PATCH /repos/:owner/:repo/hooks/:hook_id
+        url = '{}/repos/{}/{}/hooks/{}'.format(self.base_url, self.org, repo, hook_id)
+        return requests.patch(url, headers=self.headers, json=body)
+
     #
     # Teams
     #
-    def get_team(self, team, etag=None):
+    def get_team(self, team_id, etag=None):
         """Return a team of the organization."""
         # GET /teams/:team_id
-        url = '%s/teams/%s' % (self.base_url, team)
+        url = '{}/teams/{}'.format(self.base_url, team_id)
         headers = self.headers.copy()
         if etag:
             headers['If-None-Match'] = etag
-            return requests.get(url, headers=headers)
-        return requests.get(url, headers=headers).json()
+            return self.get(url, headers=headers)
+        return self.get(url, headers=headers).json()
 
     def get_team_invitations(self, team_id):
         """Return a list of team invitations."""
         # GET /teams/:team_id/invitations
         headers = {'Accept': 'application/vnd.github.dazzler-preview+json'}
-        url = 'teams/%s/invitations' % (team_id)
+        url = 'teams/{}/invitations'.format(team_id)
         return self.get_list(self.base_url, url, headers=headers)
 
     def get_team_members(self, team_id, role='all'):
         """Return a list of team members."""
         # GET /teams/:team_id/members
-        headers = {
-            # 'Accept': 'application/vnd.github.hellcat-preview+json'
-        }
+        # headers = {'Accept': 'application/vnd.github.hellcat-preview+json'}
         params = {'role': role}
-        url = 'teams/%s/members' % (team_id)
-        return self.get_list(self.base_url, url, headers=headers, params=params)
+        url = 'teams/{}/members'.format(team_id)
+        return self.get_list(self.base_url, url, params=params)
 
     def get_team_members_with_children(self, team_id, role='all'):
         """Return a list of team members."""
         # GET /teams/:team_id/members
         headers = {'Accept': 'application/vnd.github.hellcat-preview+json'}
         params = {'role': role}
-        url = 'teams/%s/members' % (team_id)
+        url = 'teams/{}/members'.format(team_id)
         return self.get_list(self.base_url, url, headers=headers, params=params)
 
     def invite_team_member(self, team, member):
         """Invite a member to the organization."""
         # PUT /teams/:id/memberships/:username
         params = {'role': 'member'}
-        url = '%s/teams/%s/memberships/%s' % (self.base_url, team, member)
+        url = '{}/teams/{}/memberships/{}'.format(self.base_url, team, member)
         return requests.put(url, headers=self.headers, params=params)
 
     def remove_team_member(self, team, member):
         """Invite a member to the organization."""
         # DELETE /teams/:id/memberships/:username
-        url = '%s/teams/%s/memberships/%s' % (self.base_url, team, member)
+        url = '{}/teams/{}/memberships/{}'.format(self.base_url, team, member)
         return requests.delete(url, headers=self.headers)
 
     def get_team_repos(self, team_id):
         """Return a list of team members."""
         # GET /teams/:team_id/repos
         # Add header to get the Nested Teams API
-        headers = {
-            # 'Accept': 'application/vnd.github.hellcat-preview+json'
-        }
-        url = 'teams/%s/repos' % (team_id)
-        return self.get_list(self.base_url, url, headers=headers)
+        # headers = {'Accept': 'application/vnd.github.hellcat-preview+json'}
+        url = 'teams/{}/repos'.format(team_id)
+        return self.get_list(self.base_url, url,)
 
     #
     # User (self - logged in user)
@@ -305,9 +329,8 @@ class GitHub(object):
     def get_self(self):
         """Return the logged in user."""
         # GET /user
-        url = '%s/user' % (self.base_url)
-        response = requests.get(url, headers=self.headers)
-        return response.content.json()
+        url = '{}/user'.format(self.base_url)
+        return self.get(url).json()
 
     def get_self_repos(self):
         """Return a list of user repos."""
@@ -317,7 +340,7 @@ class GitHub(object):
 
     def get_self_teams(self):
         """Return a list of teams for the logged-in user."""
-        # GET /user/teams
+        # GET /user/teams - requires "user" or "repo" scope
         headers = {'Accept': 'application/vnd.github.hellcat-preview+json'}
         url = 'user/teams'
         return self.get_list(self.base_url, url, headers=headers)
@@ -326,17 +349,17 @@ class GitHub(object):
     # Users
     #
     def get_user(self, login, etag=None):
-        """Return a member of the organization."""
+        """Return a single user."""
         # GET /users/:username
-        url = '%s/users/%s' % (self.base_url, login)
+        url = '{}/users/{}'.format(self.base_url, login)
         headers = self.headers.copy()
         if etag:
             headers['If-None-Match'] = etag
-            return requests.get(url, headers=headers)
-        return requests.get(url, headers=headers).json()
+            return self.get(url, headers=headers)
+        return self.get(url, headers=headers).json()
 
     def get_user_repos(self, login):
-        """Return a list of user repos."""
+        """Return a list of public repositories for the specified user."""
         # GET /users/:username/repos
-        url = 'users/%s/repos' % (login)
+        url = 'users/{}/repos'.format(login)
         return self.get_list(self.base_url, url)
